<?php
//blocking direct access the file
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

if( !class_exists('gbfdbadmin') ):
    class gbfdbadmin{
        private $settings;
        private $enabled;
        private $gbfdb_data;
        function __construct(){
            $this->settings = $this->GetGBSettings();

            add_action('admin_enqueue_scripts', array($this,"enqueue_scripts"));
            add_action("after_gbfdb_admin_page_fieldset",array($this,"after_gbfdb_admin_page_fieldset"),5);

            add_filter("gbfdb_check_lead_fields",array($this,'check_lead_fields'),10,3);
            add_filter("gbfdb_check_lead_meta",array($this,'check_lead_meta'),10,2);
            add_filter("gbfdb_check_cf7_fields",array($this,'check_cf7_fields'),10);
            add_filter("gbfdb_check_gravityforms_fields",array($this,'check_gravityforms_fields'),10);

            $this->init();

        }

        //PRIVATE

        /*
         * call : * in class
         * do : return array of settings
         */
        private function GetGBSettings(){
            $this->enabled = false;
            $out = array(
                "option" => "gbfdb_data",
                "supported_plugins" => $this->get_supported_plugins(),
                "admin-page-save" => "admin-page-save",
                "admin-page-data" => array(),
            );
            return $out;
        }

        /*
         * call : __construct
         * do : Initiate the class
         */
        private function init(){
            $this->gbfdb_data = get_option($this->settings['option']);
            if($this->gbfdb_data){
                $this->gbfdb_data = json_decode($this->gbfdb_data,true);
            }
            $admin_page_data = ($this->gbfdb_data && is_array($this->gbfdb_data) && isset($this->gbfdb_data["gbfdb-admin-option"]) ? $this->gbfdb_data["gbfdb-admin-option"] : false);
            if($admin_page_data){
                $this->settings['admin-page-data'] = $admin_page_data;

                if(isset($this->settings['admin-page-data']['gbfdb_enable']) && $this->settings['admin-page-data']['gbfdb_enable']){
                    $this->enabled = true;

                    //add the GB Forms Leeds cpt
                    if(!get_option('gbfdb_enabled')){
                        add_option('gbfdb_enabled', true);
                    }
                    //$this->EddCPT();
                }else{
                    if(get_option('gbfdb_enabled')){
                        delete_option('gbfdb_enabled');
                    }
                }
            }
        }

        /*
         * call : __construct
         * do : Initiate the class
         */
        private function get_forms_settings($forms){
            if(!$forms && !is_array($forms) && empty($forms))
                return array();

            $FrontEndData = array();

            //can mack loop with callback user function if $forms bigger then 10
            foreach ($forms as $form){
                switch ($form){
                    case "contact-form-7":
                        $FrontEndData[$form] = $this->get_cf7_settings();
                        break;
                    case "elementor":
                        $FrontEndData[$form] = $this->get_elementor_settings();
                        break;
                    case "pojo-forms":
                        $FrontEndData[$form] = $this->get_pojo_settings();
                        break;
                    default:
                        $FrontEndData[$form] = array();
                }
            }

            $FrontEndData = apply_filters("gbfdb_forms_settings",$FrontEndData);

            return $FrontEndData;
        }

        /*
         * call : get_forms_settings
         * do : collect all the cf7 settings to check the lead
         */
        private function get_cf7_settings(){
            return array(
                'type' => 'action',
                'attr' => array(
                    'name' => 'wpcf7_mail_sent',
                    'callback' => 'gbfdb_cf7_lead',
                    'priority' => 100,
                    'args' => 1
                )
            );
        }

        /*
         * call : get_forms_settings
         * do : collect all the cf7 settings to check the lead
         */
        private function get_elementor_settings(){
            return array(
                'type' => 'action',
                'attr' => array(
                    'name' => 'elementor_pro/forms/new_record',
                    'callback' => 'gbfdb_elementor_leads',
                    'priority' => 100,
                    'args' => 2
                )
            );
        }

        /*
         * call : get_forms_settings
         * do : collect all the cf7 settings to check the lead
         */
        private function get_pojo_settings(){
            return array(
                'type' => 'action',
                'attr' => array(
                    'name' => 'pojo_forms_mail_sent',
                    'callback' => 'gbfdb_pojo_forms_leads',
                    'priority' => 25,
                    'args' => 3
                )
            );
        }

        /*
         * call : in class
         * do : Load the Admin page
         */
        private function LoadAdminPage(){
            wp_enqueue_style( 'gbfdb-admin-style', GBFDBURL.'/core/css/gbfdb_admin.css');
            wp_enqueue_script( 'gbfdb-admin-script', GBFDBURL.'/core/js/gbfdb_admin.js');
            //save form data in function save_admin_data
            $valid_form = $this->gbfdc_cache_form($this->settings['admin-page-save'],array($this,'save_admin_data'));
            $gbfdb_first_activate_plugins = get_option('gbfdb_first_activate_plugins');

            //$active_plugins = get_option('active_plugins');
            $supported_plugins = $this->settings['supported_plugins'];
            $gbfdb_enable = (isset($this->settings['admin-page-data']['gbfdb_enable']) ? $this->settings['admin-page-data']['gbfdb_enable'] : false);

            $active_supported_plugins = (isset($this->settings['admin-page-data']['supported_plugins']) ? $this->settings['admin-page-data']['supported_plugins'] : array());

            //on first visit get installed forms plugins for defaults activations
            if(!$gbfdb_enable && !$active_supported_plugins){
                $active_supported_plugins = get_option('gbfdb_first_activate_plugins');
                //installed forms plugins exist then enable GBFDB as default
                if($active_supported_plugins){
                    $gbfdb_enable = true;
                }
            }

            $save_active_plugins = array();
            ?>
            <div class="gbfdb-admin-page-wrap gb_style <?php echo (empty($active_supported_plugins) ? 'empty' : ''); ?>">
                <header class="gbfdb-admin-page-header">
                    <h2><?php _e("Welcome to GB Forms DB","gb-forms-db"); ?></h2>
                    <?php if(!$gbfdb_first_activate_plugins): ?>
                    <label class="gbfdb-message error"><?php echo __('We didn&#39;t find any form plugins on your website. Please visit the settings page','gb-forms-db'); ?></label>
                    <?php endif; ?>
                    <?php if(empty($active_supported_plugins)): ?>
                    <label class="gbfdb-message error"><?php echo __('Please choose the forms plugin&#47;s you are using. If you are using a custom form&#47;s, please specify the form ID in the &#34;Custom form ID&#34; field.','gb-forms-db'); ?></label>
                    <?php endif; ?>
                </header>
                <?php
                if(is_a($valid_form, 'gbfdb_notice') && $valid_form->has_notice() > 0):
                    $has_notices = $valid_form->get_all_notices();
                    $valid_form->output_notices();
                ?>
                <?php endif; ?>
                <section class="gbfdb-admin-page-section">
                    <form method="post" id="gbfdb-admin-page">
                        <fieldset>
                            <legend>
                                <a class="gbfdb-legend selected" data-for="settings" href="#"><?php _e("Settings","gb-forms-db"); ?></a>
                                <?php do_action('gbfdb_admin_page_add_fieldset'); ?>
                            </legend>
                            <section data-item="settings" class="fields-section selected">
                                <div class="gbfdb-field-wrap">
                                    <div class="gbfdb_checkbox_wrap">
                                        <label class="switch" for="gbfdb_enable">
                                            <input type="checkbox"  id="gbfdb_enable" name="gbfdb_enable" <?php echo ($gbfdb_enable ? 'checked' : ''); ?> />
                                            <div class="slider round"></div>
                                        </label>
                                    </div>
                                    <label class="enable-disabled">
                                        <span class="input-title"><?php _e("Activate GB Forms DB","gb-forms-db") ?></span>
                                        <span class="description"><?php _e("By disabling this plugin all your leads will not be saved","gb-forms-db") ?></span>
                                        <span class="description">
                                            <?php echo sprintf(
                                                __( 'Please visit the <a target="_blank" href="%s">information page</a> to learn more', 'gb-forms-db' ),
                                                esc_url( GBFDBHOMEURL )
                                            ); ?>
                                        </span>
                                    </label>
                                </div>
                                <div class="gbfdb-fields-wrap <?php echo ($gbfdb_enable ? 'gbfdb-enable' : 'gbfdb-disable'); ?>">
                                    <div class="gbfdb-disable-wrap"></div>
                                    <?php do_action('before_gbfdb_admin_page_fields'); ?>
                                    <?php if($supported_plugins): ?>
                                        <div class="gbfdb-field-wrap">
                                            <label for="supported_plugins">
                                                <span class="input-title"><?php _e("Choose your forms","gb-forms-db"); ?></span>
                                            </label>
                                            <ul>
                                                <?php
                                                foreach ($supported_plugins as $plugin):
                                                    if($active_supported_plugins && in_array($plugin['slug'],$active_supported_plugins)){
                                                        $checked = "checked";
                                                        $save_active_plugins[] = $plugin['short'];
                                                    }else{
                                                        $checked = "";
                                                    }
                                                    if(isset($has_notices) && $valid_form->get_notice($plugin['name'])){
                                                        $checked = "";
                                                    }
                                                    ?>
                                                    <li class="gbfdb-checkbox-array <?php echo (isset($has_notices) && $valid_form->get_notice($plugin['name']) ? "not-validate" : ""); ?>">
                                                        <div class="gbfdb_checkbox_wrap small">
                                                            <label class="switch" for="supported_plugins_<?php echo esc_attr($plugin['name']); ?>">
                                                                <input type="checkbox"  id="supported_plugins_<?php echo esc_attr($plugin['name']); ?>" name="supported_plugins[]" <?php echo esc_attr($checked); ?> value="<?php echo esc_attr($plugin['slug']); ?>" />
                                                                <div class="slider round"></div>
                                                            </label>
                                                        </div>
                                                        <label>
                                                            <span><?php echo esc_html($plugin['name']); ?></span>
                                                            <?php if(isset($plugin['description']) && !empty($plugin['description'])): ?>
                                                                <div class="gbfdb-info">
                                                                    <span class="gbfdb-plug-description dashicons dashicons-info"></span>
                                                                    <p class="gbfdb-plug-description-content"><?php echo sanitize_textarea_field($plugin['description']); ?></p>
                                                                </div>
                                                            <?php endif; ?>
                                                        </label>
                                                        <?php if($plugin['field-hook']): ?>
                                                            <?php do_action($plugin['field-hook'],(isset($this->settings['admin-page-data']) ? $this->settings['admin-page-data'] : '')); ?>
                                                        <?php endif; ?>
                                                    </li>
                                                <?php endforeach; ?>
                                            </ul>
                                            <div class="message">
                                                <ul>
                                                    <li class="message-icon"><span class="dashicons dashicons-welcome-learn-more"></span></li>
                                                    <li class="message-content"><p><?php _e("All leads from enabled forms will be saved in the &#34;All Leads&#34; page","gb-forms-db"); ?></p></li>
                                                </ul>
                                            </div>
                                        </div>
                                    <?php endif; ?>
                                    <?php do_action('after_gbfdb_admin_page_fields'); ?>
                                </div>
                            </section>
                            <?php do_action('gbfdb_admin_page_add_fieldset_fields'); ?>
                        </fieldset>
                        <div class="gbfdb-submit-wrap">
                            <button class="button button-primary" type="submit"><?php _e('Save'); ?></button>
                        </div>
                    </form>
                </section>
            </div>
            <?php
            if($save_active_plugins && !empty($save_active_plugins)){
                update_option("gbfdp_supported_plugins",$save_active_plugins);
            }
        }


        //PUBLIC

        /*
         * call : __construct
         * do : add all the hooks and filter
         */
        public function hooks(){

        }

        /*
         * call : __construct
         * do : add all the hooks and filter
         */
        public function enqueue_scripts($page){
            if (strpos($page, 'gbformsdb') !== false){
                wp_enqueue_style( 'gbweb-block-ui-style', GBFDBURL.'/core/css/gbweb-block-ui.css',GBFDBV);
                wp_enqueue_script( 'gbweb-block-ui-script', GBFDBURL.'/core/js/gbweb-block-ui.js',GBFDBV);

                wp_enqueue_style( 'gbfdb-admin-style', GBFDBURL.'/core/css/gbfdb_admin.css',array('gbweb-block-ui-style'),GBFDBV);
                wp_enqueue_script( 'gbfdb-admin-script', GBFDBURL.'/core/js/gbfdb_admin.js',array('gbweb-block-ui-script'),GBFDBV);
                wp_localize_script( 'gbfdb-admin-script', 'gbfdb_api', array(
                    'ajaxurl' => admin_url( 'admin-ajax.php' ),
                    'api_sod' => wp_create_nonce( 'api_sod' )
                ));

                wp_enqueue_script('jquery-ui-datepicker');
            }
        }

        /*
         * call : *
         * do : return array of supported plugins
         */
        public function get_supported_plugins(){
            if(!isset($this->settings['supported_plugins'])){
                $supported_plugins = get_option("gbfdb_supported_plugins");
            }else{
                $supported_plugins = $this->settings['supported_plugins'];
            }
            if($supported_plugins){
                $supported_plugins = apply_filters("gbfdb_supported_plugins",$supported_plugins);
            }
            return $supported_plugins;
        }

        /*
         * call : *
         * do : return array of supported plugins
         */
        public function save_admin_data(){
            //die(print_r($_POST,true));

            if($_POST && isset($_POST[$this->settings['admin-page-save']])){

                //validate data to save
                $supported_plugins = $this->get_supported_plugins();

                if($supported_plugins){
                    $gbfdb_notice = new gbfdb_notice();
                    foreach ($supported_plugins as $plugin){
                        if(isset($plugin['validation_hook']) && !empty($plugin['validation_hook'])){
                            if(isset($_POST['supported_plugins']) && !empty($_POST['supported_plugins']) && in_array($plugin['slug'],$_POST['supported_plugins'])){
                                $message = apply_filters($plugin['validation_hook'],'',$_POST);
                                if(is_string($message) && !empty($message)){
                                    $gbfdb_notice->add_notice($plugin['name'],$message);
                                }
                            }
                        }
                    }

                    if($gbfdb_notice->has_notice()){
                        return $gbfdb_notice;
                    }
                }

                unset($_POST[$this->settings['admin-page-save']]);
                $temp_post = apply_filters("gbfdb-before-admin-save",$_POST);

                $gbfdb_data = array(
                   'gbfdb-admin-option' => $temp_post,
                    "gbfdb_supported_plugins" => $supported_plugins,
                    "frontend_data" => array(
                        "supported_plugins" => (isset($_POST["supported_plugins"]) && !empty($_POST["supported_plugins"]) ? $_POST["supported_plugins"] : array())
                    )
                );
                if(!empty($gbfdb_data['frontend_data']['supported_plugins'])){
                    $gbfdb_data['frontend_data']['supported_plugins'] = $this->get_forms_settings($gbfdb_data['frontend_data']['supported_plugins']);
                    $gbfdb_data = apply_filters("gbfdb-admin-save",$gbfdb_data);
                }
                if(update_option($this->settings['option'],json_encode($gbfdb_data))){
                    $this->settings['admin-page-data'] = $_POST;
                }
                $this->init();
            }
            return true;
        }

        /*
         * call : do_action('after_gbfdb_admin_page_fieldset');
         * do : add fields to the end of gbfdb admin page form
         */
        public function after_gbfdb_admin_page_fieldset(){
            ?>
            <input type="hidden" name="<?php echo (isset($this->settings) && isset($this->settings["admin-page-save"]) ? esc_attr($this->settings["admin-page-save"]) : 'untitled'); ?>" value="1">
            <?php
        }

        /*
         * call : *
         * do :
         */
        public function gbfdc_cache_form($field_name,$callback){
            if(empty($field_name) && empty($callback))
                return false;

            if(isset($_POST) && isset($_POST[$field_name])){
                if(is_callable($callback)){
                    return call_user_func($callback);
                }
            }
            return false;
        }

        /*
         * call : *
         * do : call to LoadAdminPage
         */
        public function AdminPage(){
            $this->LoadAdminPage();
        }

        /*
         * call : add_filter("gbfdb_check_lead_fields",array($this,'check_lead_fields'),10,3);
         * do : call filters to check the lead fields
         */
        public function check_lead_fields($fields,$name,$type){
            if($this->settings['supported_plugins'] && !empty($type)){

                foreach ($this->settings['supported_plugins'] as $supported_plugin){
                    if($type && strpos($type,$supported_plugin['short']) !== false){
                        $fields = apply_filters("gbfdb_check_{$supported_plugin['short']}_fields",$fields);
                    }
                }
            }

            return $fields;
        }

        /*
         * call : add_filter("gbfdb_check_lead_meta",array($this,'check_lead_fields'),10,2);
         * do : call filters to check the lead meta
         */
        public function check_lead_meta($meta,$name){
            if(is_array($meta)){
                foreach ($meta as $key => $value){
                    if($key == "timestamp"){
                        $meta[$key] = date(GBFDBDATAFORMAT, $value);
                    }
                }
            }
            return $meta;
        }

        /*
         * call : add_filter("gbfdb_check_cf7_fields",array($this,'check_cf7_fields'),10)
         * do : check the cf7 fields
         */
        public function check_cf7_fields($fields){
            foreach ($fields as $key => $val){
                if(substr($key,0,1) == '_'){
                    unset($fields[$key]);
                }
            }
            return $fields;
        }

        /*
         * call : add_filter("gbfdb_check_gravityforms_fields",array($this,'check_gravityforms_fields'),10)
         * do : check the cf7 fields
         */
        public function check_gravityforms_fields($fields){
            foreach ($fields as $key => $val){
                if(strpos($key, "input_") === false){
                    unset($fields[$key]);
                }
            }
            return $fields;
        }
    }
//Return the gbfdbadmin class object
    function gbfdbadmin(){
        global $gbfdbadmin;

        if( !isset($gbfdbadmin) ){
            $gbfdbadmin = new gbfdbadmin();
        }

        return $gbfdbadmin;
    }

//End fo "do only if class NOT exists"
endif;

gbfdbadmin();