<?php
//blocking direct access the file
defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

if( !class_exists('gbfdbfront') ):
    class gbfdbfront{
        private $settings;
        private $enabled;
        private $FrontData;
        function __construct(){
            $this->settings = $this->GetGBSettings();

            add_action("wp_enqueue_scripts",array($this,"LoadFront"));

            $this->init();

        }

        //PRIVATE

        /*
         * call : * in class
         * do : return array of settings
         */
        private function GetGBSettings(){
            $data = get_option("gbfdb_data");
            if($data){
                $data = json_decode($data,true);
            }
            $out = array(
                'supported_plugins' => ($data && isset($data["gbfdb_supported_plugins"]) ? $data["gbfdb_supported_plugins"] : false),
                'admin-page-option' => 'gbfdb-admin-option',
                'admin-page-data' => ($data && isset($data["gbfdb-admin-option"]) ? $data["gbfdb-admin-option"] : false),
            );

            $this->FrontData = ($data && isset($data["frontend_data"]) ? $data["frontend_data"] : "#");
            if(isset($out['admin-page-data']) && isset($out['admin-page-data']['gbfdb-form-selector']) && $out['admin-page-data']['gbfdb-form-selector'] != '' && isset($this->FrontData['supported_plugins']) && isset($this->FrontData['supported_plugins']['form'])){
                $this->FrontData['supported_plugins']['form']['gbfdb-form-selector'] = $out['admin-page-data']['gbfdb-form-selector'];
            }

            return $out;
        }

        /*
         * call : __construct
         * do : Initiate the class
         */
        private function init(){
            if(!empty($this->settings["admin-page-data"]) && $this->settings["admin-page-data"]){
                if(isset($this->settings["admin-page-data"]['gbfdb_enable']) && $this->settings["admin-page-data"]['gbfdb_enable'] == 'on'){
                    $this->enabled = true;
                }else{
                    $this->settings["admin-page-data"] = false;
                    $this->enabled = false;
                }
            }
        }

        //PUBLIC

        /*
         * call : add_action("wp_enqueue_scripts",array($this,"LoadFront"))
         * do : load front end script and css
         */
        public function LoadFront(){
            if(!$this->enabled && !is_array($this->FrontData))
                return false;

            $data = array(
                'ajaxurl' => admin_url( 'admin-ajax.php' ),
                'sod' => wp_create_nonce( 'gbfdb_sod' ),
                'data' => $this->FrontData
            );

            wp_enqueue_script("gbfdb-front-js",GBFDBURL."/core/js/gbfdb_front.js","",true);
            wp_localize_script( 'gbfdb-front-js', 'gb-forms-db', $data );
            return true;
        }

        public function check_form_action($type, $hook){
            if($this->FrontData && isset($this->FrontData['supported_plugins'])){
                foreach ($this->FrontData['supported_plugins'] as $supported_plugin){
                    if(is_array($supported_plugin) && !empty($supported_plugin)){
                        if($type == $supported_plugin['type'] && isset($supported_plugin['attr']) && $hook == $supported_plugin['attr']['name']){
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        /*
         * call : *
         * do : call to LoadAdminPage
         */
        public function get_supported_plugins(){
            if($this->settings["supported_plugins"]){
                return $this->settings["supported_plugins"];
            }
            return false;
        }
    }
//Return the gbfdbfront class object
    function gbfdbfront(){
        global $gbfdbfront;

        if( !isset($gbfdbfront) ){
            $gbfdbfront = new gbfdbfront();
        }

        return $gbfdbfront;
    }

//End fo "do only if class NOT exists"
endif;

gbfdbfront();